import { SignalSourceEntity } from "./SignalSourceEntity";
import ApplicationConfigEntity from "./ApplicationConfigEntity";

export namespace Protocol {
  export class Commands {
    public static get PROTOCOL_PREFIX() {
      return "_";
    }

    public static get kUnKnowCommand() {
      return Commands.PROTOCOL_PREFIX + "UnKnowCommand";
    }

    public static get kSearchDevice() {
      return Commands.PROTOCOL_PREFIX + "SearchDevice";
    }
    public static get kEditNetworkInterfaceInfo() {
      return Commands.PROTOCOL_PREFIX + "EditNetworkInterfaceInfo";
    }
    public static get kExecCommand() {
      return Commands.PROTOCOL_PREFIX + "ExecCommand";
    }
    public static get kRestoreDevice() {
      return Commands.PROTOCOL_PREFIX + "RestoreDevice";
    }
    public static get kLogin() {
      return Commands.PROTOCOL_PREFIX + "Login";
    }
    public static get kLogout() {
      return Commands.PROTOCOL_PREFIX + "Logout";
    }
    public static get kRpcGetWindows() {
      return Commands.PROTOCOL_PREFIX + "RpcGetWindows";
    }
    public static get kRpcGetSignalSources() {
      return Commands.PROTOCOL_PREFIX + "RpcGetSignalSources";
    }

    public static get kRpcGetApplicationConfig() {
      return Commands.PROTOCOL_PREFIX + "RpcGetApplicationConfig";
    }

    static _all_commands = new Set([
      Commands.kUnKnowCommand,
      Commands.kSearchDevice,
      Commands.kEditNetworkInterfaceInfo,
      Commands.kExecCommand,
      Commands.kRestoreDevice,
      Commands.kLogin,
      Commands.kLogout,
      Commands.kRpcGetWindows,
      Commands.kRpcGetSignalSources,
      Commands.kRpcGetApplicationConfig,
    ]);

    public static get AllCommands() {
      return this._all_commands;
    }
  }

  export class PacketEntity {
    public static get FLAG_REQUEST() {
      return 0;
    }
    public static get FLAG_RESPONSE() {
      return 1;
    }
    public static get FLAG_NOTIFY() {
      return 2;
    }

    /** 是否出现异常,如果是异常,则为 ExceptionEntity */
    has_exception = false;
    /** 0: 请求  1: 响应 */
    flag = 0;
    /** rpc 消息 ID，默认 0 */
    rpc_id = 0;
    /** 命令 */
    command = Protocol.Commands.kUnKnowCommand;
  }

  export class LoginRequest extends PacketEntity {
    user_name = "";
    password = "";

    constructor(user_name?: string, password?: string) {
      super();
      this.user_name = user_name ?? "";
      this.password = password ?? "";
      this.command = Commands.kLogin;
    }
  }

  export class LoginResponse extends PacketEntity {
    success = false;
    error_code = 0;

    constructor() {
      super();
      this.command = Commands.kLogin;
    }
  }

  export class GetSignalSourcesRequest extends PacketEntity {
    timestamp = new Date().getMilliseconds();

    constructor(rpcid?: number) {
      super();
      this.rpc_id = rpcid ?? 0;
      this.command = Commands.kRpcGetSignalSources;
    }
  }

  export class GetSignalSourcesResponse extends PacketEntity {
    signal_sources: SignalSourceEntity[] = [];
    signal_source_groups: [] = [];

    constructor() {
      super();
      this.command = Commands.kRpcGetSignalSources;
    }
  }

  export class GetApplicationConfigRequestEntity extends PacketEntity {
    timestamp: number = new Date().getMilliseconds();

    constructor(rpcid?: number) {
      super();
      this.rpc_id = rpcid ?? 0;
      this.command = Commands.kRpcGetApplicationConfig;
    }
  }

  export class GetApplicationConfigResponseEntity extends PacketEntity {
    config: ApplicationConfigEntity | null = null;

    constructor() {
      super();
      this.command = Commands.kRpcGetApplicationConfig;
    }
  }
}
